--
-- Mod: Animal Worker
--
-- @Author: Tunnelrat
-- @Website: TRmods.com or https://mods.to/users/Tunnelrat
-- @Email: tunnelratdev@gmail.com
-- @Date: 06.25.2019
-- @Version: 2.3
-- Copyright (C) Tunnelrat, All Rights Reserved.

--[[
	Change log:
	Version 2.3:
	- GUI now loads with current settings
	- Help menu text displays whether worker is active or not
	- On-screen feedback (top notification) when worker toggled or charges for work, shows for 5 seconds
	- Now trains horses 4 times a day with configurable hourly rate per horse at 8am, 9am, 10am, 11am (if you have HorseHelper, set train Horses to false)
	- More settings configurations
	
	TO DO:
	 - More language settings (g_i18n variables)
	 - Add controller inputs
	 - Add option to sell eggs, milk, manure, slurry, wool
]]--

animalWorker = {};

local modDesc = loadXMLFile("modDesc", g_currentModDirectory .. "modDesc.xml")

-- global values for settings
animalWorkerActive = true -- if true, the Ranch Hand will work everyday, set to false to disable. Default: true
animalWorkerFee = 0.05 -- convenience fee percentage for doing all this work. Default: 0.05
costPerAnimal = 10.00 -- base cost to tend to each animal. Default: 10.00
fillToCapacity = true -- false if you want to give enough food,water,straw for 6 days. Default: true
cleanPens = true -- if true, will also clean the animal pens once a day. Default: true
horseRideActive = true -- if true, worker will ride each horse 25% of DAILY_TARGET_RIDING_TIME for 4 hours (between 0800 and 1100)
horseRideRate = 37.50 -- hourly rate to train each horse (animalWorkerFee does not get added) (Default: 37.50 per hour * 4 = 150 per horse per day)

-- Place holder
--helpDisplay = "awActive" -- nothing yet, place holder for messages for now, ignore

-- Notification parameters
nText 	= ""	-- init var (str)
nDur	= 5000 -- 5000 ms or 5 sec (int)
counter = 0		-- for display durations
sendIt = false

-- Text display settings
--[[
 You can change these if you want.
 x = 0.00 is the left of screen, x = 1.00 is the right of screen
 y = 0.00 is the bottom of screen, y = 1.00 is the top of screen
 setTextBold(true) = bold text; false for no bold text
 alignment options are RenderText.ALIGN_LEFT | RenderText.ALIGN_CENTER | RenderText.ALIGN_RIGHT
]]--
fontSize 	= 0.02
posX		= 0.30
posY		= 0.92
setTextBold(true)	-- bold
setTextColor(1, 0.5, 0.1, 1) -- initial color is red / orange
setTextAlignment(RenderText.ALIGN_CENTER)	-- align center

animalWorker.version = getXMLString(modDesc, "modDesc.version")
animalWorker.modDirectory = g_currentModDirectory

animalWorker.fees = {0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, 0.50}
animalWorker.costs = {1.00, 2.50, 5.00, 10.00, 15.00, 20.00, 25.00, 50.00, 100.00, 250.00}
animalWorker.rates = {12.50, 18.75, 25.00, 31.25, 35.00, 37.50, 40.00, 43.75, 50.00, 75.00}

addModEventListener(animalWorker)

function animalWorker:loadMap()	

	print("###*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*##")
	print("##                Animal Worker v " .. self.version .. "	                   ##")
	print("##		      by Tunnelrat - TRmods.com     			   ##")
	print("###*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*##")

	-- Settings
	animalWorker.settingsDir = getUserProfileAppPath().. "modsSettings/animalWorker/"
	
	-- Init data tables
	animalWorker.eventName = {}
	animalWorker.horse = {}

	-- Gui Class inheritance
	animalWorker.gui = {}
	animalWorker.gui["awSettingGui"] = awGui:new()
	g_gui:loadGui(animalWorker.modDirectory .. "gui/awGui.xml", "awGui", animalWorker.gui.awSettingGui)

	-- Load Mod Settings
	animalWorker:LoadSettings()
	animalWorker:SaveSettings()
	animalWorker:updateGui()
	
	-- Player action event registration
	Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, animalWorker.registerActionEventsPlayer)
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, animalWorker.removeActionEventsPlayer)

end

function animalWorker:SaveSettings()

	createFolder(getUserProfileAppPath() .. "modsSettings/")
	createFolder(animalWorker.settingsDir)
	local file = animalWorker.settingsDir .. "animalWorker.xml"
	local pos = 0
	local rootPath = "animalWorker.settings"
	local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)

	if not fileExists(file) then
		-- create
		local xmlContent = createXMLFile("animalWorker_XML", file, "animalWorker")
		
		setXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive", animalWorkerActive)
		setXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee", animalWorkerFee)
		setXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal", costPerAnimal)
		setXMLBool(xmlContent, groupNameTag .. "#fillToCapacity", fillToCapacity)
		setXMLBool(xmlContent, groupNameTag .. "#cleanPens", cleanPens)
		setXMLBool(xmlContent, groupNameTag .. "#horseRideActive", horseRideActive)
		setXMLFloat(xmlContent, groupNameTag .. "#horseRideRate", horseRideRate)
		saveXMLFile(xmlContent)
		delete(xmlContent)
	else
		-- update
		local xmlContent = createXMLFile("animalWorker_XML", file, "animalWorker")
		
		setXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive", animalWorkerActive)
		setXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee", animalWorkerFee)
		setXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal", costPerAnimal)
		setXMLBool(xmlContent, groupNameTag .. "#fillToCapacity", fillToCapacity)
		setXMLBool(xmlContent, groupNameTag .. "#cleanPens", cleanPens)
		setXMLBool(xmlContent, groupNameTag .. "#horseRideActive", horseRideActive)
		setXMLFloat(xmlContent, groupNameTag .. "#horseRideRate", horseRideRate)
		saveXMLFile(xmlContent)
		delete(xmlContent)
	end

end

function animalWorker:LoadSettings()
	
	local file = animalWorker.settingsDir .. "animalWorker.xml"
	
	if fileExists(file) then
		local xmlContent = loadXMLFile("animalWorker_XML", file, "animalWorker")
		
		local pos = 0
		local rootPath = "animalWorker.settings"
		local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)
		
		animalWorkerActive 	= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive"), true)
		animalWorkerFee		= Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee"), 0.05)
		costPerAnimal		= Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal"), 10.00)
		fillToCapacity		= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#fillToCapacity"), true)
		cleanPens			= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#cleanPens"), true)
		horseRideActive		= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#horseRideActive"), true)
		horseRideRate		= Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#horseRideRate"), 37.50)
	else
		print("Animal Worker XML settings not found.")
	end
	
end

function animalWorker:delete()

end

function animalWorker:registerActionEventsPlayer()
	-- Animal Worker Toggle (true/false)
	local r, eventName = InputBinding.registerActionEvent(g_inputBinding, 'animalWorker_Active', self, animalWorker.actionAnimalWorkerActive, false, true, false, true)
	if r then
		table.insert(animalWorker.eventName, eventName)
		g_inputBinding.events[eventName].displayIsVisible = true
		g_inputBinding:setActionEventText(eventName, g_i18n:getText("input_animalWorker_Active"))
    end
	-- Animal Worker Settings
	local r, eventName = InputBinding.registerActionEvent(g_inputBinding, 'animalWorker_Setting', self, animalWorker.actionAnimalWorker_Setting, false, true, false, true)
	if r then
		table.insert(animalWorker.eventName, eventName)
		g_inputBinding.events[eventName].displayIsVisible = true
		g_inputBinding:setActionEventText(eventName, g_i18n:getText("input_animalWorker_Setting"))
	end
end

function animalWorker:removeActionEventsPlayer()
	animalWorker.eventName = {}
end

function animalWorker:actionAnimalWorkerActive(actionName)
	if actionName == "animalWorker_Active" then -- key pressed
		if animalWorkerActive then
			animalWorkerActive = false
			nText = g_i18n:getText("awNotActive")
			g_inputBinding:setActionEventText(actionName, g_i18n:getText("awNotActive"))
			print("Animal Worker: Debug Note Text = " .. nText)
		elseif not animalWorkerActive then
			animalWorkerActive = true
			nText = g_i18n:getText("awActive")			
			g_inputBinding:setActionEventText(actionName, g_i18n:getText("awActive"))
			print("Animal Worker: Debug Note Text = " .. nText)
		end
		-- Start text display counter with nText
		counter = nDur
	end
	animalWorker:SaveSettings()
	print("Animal Worker: Saving Settings")
end

-- Show settings menu
function animalWorker:actionAnimalWorker_Setting(actionName, keyStatus, arg3, arg4, arg5)
	if animalWorker.gui.awSettingGui.isOpen then
		animalWorker.gui.awSettingGui:onClickBack()
	elseif g_gui.currentGui == nil then
		g_gui:showGui("awGui")
	end
end

function animalWorker:keyEvent(unicode, sym, modifier, isDown)

end

function animalWorker:mouseEvent(posX, posY, isDown, isUp, button)
end

function animalWorker:deleteMap()
end

function animalWorker:updateGui()
	-- Worker Status
	animalWorker.gui.awSettingGui:setActiveStatus(animalWorkerActive)
	
	-- Worker Fee
	index = 0
	for i = 1, #animalWorker.fees, 1 do
		if (animalWorker.fees[i] == animalWorkerFee) then
			index = i
			break
		end
	end
	if (index ~= 0) then
		animalWorker.gui.awSettingGui:setWorkerFee(index)
	end
	
	-- Worker Cost
	index = 0
	for i = 1, #animalWorker.costs, 1 do
		if (animalWorker.costs[i] == costPerAnimal) then
			index = i
			break
		end
	end
	if (index ~= 0) then
		animalWorker.gui.awSettingGui:setWorkerCost(index)
	end
	
	-- Worker Capacity
	animalWorker.gui.awSettingGui:setCapacity(fillToCapacity)
	
	-- Worker Pens
	animalWorker.gui.awSettingGui:setCleanPens(cleanPens)
	
	-- Train Horses
	animalWorker.gui.awSettingGui:setTrainHorse(horseRideActive)
	
	-- Train Horses Hourly Rate
	index = 0
	for i = 1, #animalWorker.rates, 1 do
		if (animalWorker.rates[i] == horseRideRate) then
			index = i
			break
		end
	end
	if (index ~= 0) then
		animalWorker.gui.awSettingGui:setHorseRate(index)
	end
end

function animalWorker:settingsFromGui(sActive, sFee, sCost, sCap, sPen, sTrain, sRate)
	-- Update settings
	animalWorkerActive = sActive
	animalWorkerFee = animalWorker.fees[sFee]
	costPerAnimal = animalWorker.costs[sCost]
	fillToCapacity = sCap
	cleanPens = sPen
	horseRideActive = sTrain
	horseRideRate = sRate
	-- Save to xml
	animalWorker:SaveSettings()
	animalWorker:LoadSettings()
end

function animalWorker:addNote(text, count, visible)
	dText = tostring(text)
	dCount = count or counter
	local isVis = "Yes"
	if dCount <= 0 then
		visible = false
		isVis = "No"
	end
	if not visible then
		text = ""
	end
	-- render display text
	if visible then
		renderText(posX, posY, fontSize, dText)
	end
end

function animalWorker:update(dt)
	
	local hourNow = g_currentMission.environment.currentHour
	local minuteNow = g_currentMission.environment.currentMinute
		
	if counter > 0 then
		counter = counter - dt
		animalWorker:addNote(nText, counter, true)
	end
	
	-- Animal Worker Status Help Text (bottom of F1 help menu)
	local showStatus = g_i18n:getText("awActive")
	if not animalWorkerActive then
		showStatus = g_i18n:getText("awNotActive")
	end
	g_currentMission:addExtraPrintText(showStatus)
	
	farmData = {
		ownerFarmId = 0,
		farmId = 0,
	}
	
	local playerOwnsFarm = false
	
	function checkIsOwnerOfFarm(husbandry)
		if g_currentMission:getIsClient() then
			farmData.ownerFarmId = g_currentMission.player.ownerFarmId
			farmData.farmId = g_currentMission.player.farmId
			
			if husbandry.modulesByName.animals.owner.ownerFarmId == farmData.farmId then
				playerOwnsFarm = true
				return true
			else
				playerOwnsFarm = false
				return false
			end
		end
	end
	
	function animalCount(husbandry)
		if husbandry ~= nil then
			local animals = husbandry.modulesByName.animals.animals
			local numAnimals = 0
			for k,v in pairs(animals) do
				numAnimals = tonumber(k)
			end
			return numAnimals
		else
			print("animalWorker:bad parameters for animalCount()")
			return 0
		end
	end
	--[[FillTypes concatenated
	local wheat = 2	local barley = 3	local canola = 5	local forage = 19	local silage = 24	local hay = 30
	local grass = 28	local sunflower = 6	local soybean = 7 local maize = 8	local oat = 4	local potato = 9	local sugerbeet = 10]]--
	local chickens, cows, horse, pigs, sheep, all = 0, 0, 0, 0, 0, 0
	local eatDayChicken = 5
	local eatDayCow = 350
	local eatDayHorse = 400
	local eatDayPig = 90
	local eatDaySheep = 50
	-- Init horse riding timer and delta
	local horseRideTimer = 0.00 -- initial ride timer value
	local horseDelta = 0.25 -- Add 25% to ride timer each hour for 4 hours
	
	-- Fill water troughs, food and straw at 7am if worker is active
	if hourNow == 7 and minuteNow == 00 and animalWorkerActive and not self.doOnce then
		
		-- Run one time
		self.doOnce = true
		-- FOR LOOP
		for _,husbandry in pairs(g_currentMission.husbandries) do
		
			local animalType = husbandry.modulesByName.animals.animalType
		
			if checkIsOwnerOfFarm(husbandry) == true then
				
				-- Clean Pens
				if cleanPens == true then
					if husbandry.modulesByName.foodSpillage ~= nil then
						husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
						husbandry:setModuleParameters("foodSpillage", 0, 0)
					end
				end
					
				-- Water
				if husbandry.modulesByName.water ~= nil then
					local amountToAdd = husbandry.modulesByName.water:getFreeCapacity(FillType.WATER)
					local fillLevel, capacity = 0, 0
    
					fillLevel = husbandry.modulesByName.water:getFillLevel(FillType.WATER)
					capacity = husbandry.modulesByName.water.fillCapacity
		
					if amountToAdd > 0 then
						if fillToCapacity == true then
							husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)					
						else
							local fillTo = 0
							if animalType ~= nil and animalType == "CHICKEN" then -- base game gives water consumption amount, use for mods/updates
								chickens = animalCount(husbandry)
								fillTo = tonumber(math.ceil(chickens * 7 * 6)) -- 7 L per chicken (6 days)
							elseif animalType ~= nil and animalType == "COW" then
								cows = animalCount(husbandry)
								fillTo = tonumber(math.ceil(cows * 80 * 6)) -- 80 L per cow (6 days)
							elseif animalType ~= nil and animalType == "HORSE" then
								horse = animalCount(husbandry)
								fillTo = tonumber(math.ceil(horse * 50 * 6)) -- 50 L per horse (6 days)
							elseif animalType ~= nil and animalType == "PIG" then
								pigs = animalCount(husbandry)
								fillTo = tonumber(math.ceil(pigs * 20 * 6)) -- 20 L per pig (6 days)
							elseif animalType ~= nil and animalType == "SHEEP" then
								sheep = animalCount(husbandry)
								fillTo = tonumber(math.ceil(sheep * 20 * 6)) -- 20 L per sheep (6 days)
							end
							husbandry.modulesByName.water:setFillLevel(FillType.WATER, fillTo)
						end
					end
				end
				
				-- Straw
				if husbandry.modulesByName.straw ~= nil then
					local amountToAdd = husbandry.modulesByName.straw:getFreeCapacity(FillType.STRAW)
					local fillLevel, capacity = 0, 0
    
					fillLevel = husbandry.modulesByName.straw:getFillLevel(FillType.STRAW)
					capacity = husbandry.modulesByName.straw.fillCapacity
		
					if amountToAdd > 0 then
						if fillToCapacity == true then
							husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, capacity)					
						else
							local fillTo = 0
							if animalType ~= nil and animalType == "CHICKEN" then
								-- Do nothing, 0 straw
							elseif animalType ~= nil and animalType == "COW" then
								cows = animalCount(husbandry)
								fillTo = tonumber(math.ceil(cows * 95 * 6)) -- 95 L per cow (6 days)
							elseif animalType ~= nil and animalType == "HORSE" then
								horse = animalCount(husbandry)
								fillTo = tonumber(math.ceil(horse * 80 * 6)) -- 80 L per horse (6 days)
							elseif animalType ~= nil and animalType == "PIG" then
								pigs = animalCount(husbandry)
								fillTo = tonumber(math.ceil(pigs * 30 * 6)) -- 30 L per pig (6 days)
							elseif animalType ~= nil and animalType == "SHEEP" then
								-- Do nothing, 0 straw
							end
							husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, fillTo)
						end
					end
				end
				
				-- Chicken
				if animalType ~= nil and animalType == "CHICKEN" then
					chickens = animalCount(husbandry)
					
					-- Food					
					if chickens > 0 then
						local foodToAdd = 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(chickens * 5 * 6)) -- numAnimals * litersFoodPerAnimal * dayToInterval
						else
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " wheat and barley to " .. chickens .. " chickens.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.WHEAT, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.BARLEY, foodToAdd)
						end
					end
				end	-- End chicken
				
				-- Cow
				if animalType ~= nil and animalType == "COW" then
					cows = animalCount(husbandry)
					
					-- Food
					if cows > 0 then
						local foodToAdd, f75, f25 = 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(cows * 500 * 6))
							f75 = tonumber(math.floor(cows * math.ceil(350 * 0.75) * 6)) -- 75% eat weight
							f25 = tonumber(math.floor(cows * math.ceil(350 * 0.25) * 6)) -- 25%
						else
							f75 = tonumber(math.ceil(fCap * 0.75))
							f25 = tonumber(math.ceil(fCap * 0.25))
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " TMR, " .. f75 .. " silage and hay, " .. f25 .. " grass to " .. cows .. " cows.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.FORAGE, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.SILAGE, foodToAdd) --f75)
							husbandry.modulesByName.food:setFillLevel(FillType.DRYGRASS_WINDROW, foodToAdd) --f75)
							husbandry.modulesByName.food:setFillLevel(FillType.GRASS_WINDROW, foodToAdd) --f25)
						end
					end
					
				end -- End cow
				
				-- Horse
				if animalType ~= nil and animalType == "HORSE" then
					horse = animalCount(husbandry)
					
					-- Food
					if horse > 0 then
						local foodToAdd, f60, f40 = 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(horse * 400 * 6))
							f60 = tonumber(math.floor(horse * math.ceil(400 * 0.6) * 6)) -- 60% eat weight
							f40 = tonumber(math.floor(horse * math.ceil(400 * 0.4) * 6)) -- 40% eat weight
						else
							f60 = tonumber(math.ceil(fCap * 0.6))
							f40 = tonumber(math.ceil(fCap * 0.4))
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " oats and hay to " .. horse .. " horses.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.OAT, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.DRYGRASS_WINDROW, foodToAdd)
						end
					end
				end -- End horse
				
				-- Pig
				if animalType ~= nil and animalType == "PIG" then
					pigs = animalCount(husbandry)
					
					-- Food
					if pigs > 0 then
						local foodToAdd, f50, f25, f20, f5 = 0, 0, 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(pigs * 90 * 6)) -- 100%
							f50 = tonumber(math.floor(pigs * math.ceil(90 * 0.5) * 6)) -- 50%
							f25 = tonumber(math.floor(pigs * math.ceil(90 * 0.25) * 6)) -- 25%
							f20 = tonumber(math.floor(pigs * math.ceil(90 * 0.2) * 6)) -- 20%
							f5 = tonumber(math.floor(pigs * math.ceil(90 * 0.05) * 6)) -- 5%
						else
							f50 = tonumber(math.ceil(fCap * 0.5))
							f25 = tonumber(math.ceil(fCap * 0.25))
							f20 = tonumber(math.ceil(fCap * 0.2))
							f5 = tonumber(math.ceil(fCap * 0.05))
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed (" .. foodToAdd .. " total) " .. f50 .. " corn, " .. f25 .. " grains, " .. f20 .. " proteins, " .. f5 .. " roots to " .. pigs .. " pigs.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.MAIZE, foodToAdd) --f50)
							husbandry.modulesByName.food:setFillLevel(FillType.WHEAT, foodToAdd) --f25)
							husbandry.modulesByName.food:setFillLevel(FillType.BARLEY, foodToAdd) --f25)
							husbandry.modulesByName.food:setFillLevel(FillType.SOYBEAN, foodToAdd) --f20)
							husbandry.modulesByName.food:setFillLevel(FillType.CANOLA, foodToAdd) --f20)
							husbandry.modulesByName.food:setFillLevel(FillType.SUNFLOWER, foodToAdd) --f20)
							husbandry.modulesByName.food:setFillLevel(FillType.POTATO, foodToAdd) --f5)
							husbandry.modulesByName.food:setFillLevel(FillType.SUGARBEET, foodToAdd) --f5)
						end
					end
				end -- End pig
				
				-- Sheep
				if animalType ~= nil and animalType == "SHEEP" then
					sheep = animalCount(husbandry)
					-- Food
					if sheep > 0 then
						local foodToAdd = 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(sheep * 50 * 6)) -- 100%
						else
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " grass to " .. sheep .. " sheep.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.GRASS_WINDROW, foodToAdd)
						end
					end
				end -- End sheep
			end -- owner
		
		end -- end for loop
		
		all = tonumber(chickens + cows + pigs + horse + sheep)
		
		-- Pay the worker...
		local difficultyMultiplier = g_currentMission.missionInfo.buyPriceMultiplier
		local moneyUnit = g_gameSettings.moneyUnit
		local showCurrency = "DOLLARS"
		if moneyUnit == "1" then
			showCurrency = "EUROS"
		elseif moneyUnit == "2" then
			showCurrency = "DOLLARS"
		elseif moneyUnit == "3" then
			showCurrency = "POUNDS"
		end
		local price = 0
		local farmCost = 0
		if all ~= 0 then
			price = tonumber(math.ceil(costPerAnimal * all * difficultyMultiplier)) -- 10 * numAnimals * {1.0, 1.8, 3.0}
			price = price + (math.floor(price * animalWorkerFee)) -- sum with fee
			farmCost = tonumber(math.floor(price))
			-- print("animalWorker:Cost " .. farmCost .. " total.")
			if farmCost ~= 0 then
				g_currentMission:farmStats(farmData.farmId):updateStats("expenses", farmCost)
				g_currentMission:addMoney(-farmCost, farmData.farmId, MoneyType.ANIMAL_UPKEEP, true)
				nText = "Animal Worker: (-" .. farmCost .. " " .. showCurrency .. ") " .. all .. " Animals."
				-- Display Update
				counter = nDur
				print("Animal Worker: Ranch hand fed, watered and bedded all " .. all .. " animals at 7am for " .. farmCost .. " " .. showCurrency .. ".")
				self.doOnce = false
			end
		end	-- End costs
		
	end
	
	-- Work around for horse loops - set doHorse false every hour (note: find a better way)
	if self.doHorse and minuteNow == 01 then
		self.doHorse = false
	end
	
	-- Ride horses if active
	if horseRideActive and not self.doHorse and minuteNow == 00 and (hourNow == 8 or hourNow == 9 or hourNow == 10 or hourNow == 11) then
		
		local c = 0
		
		for _, husbandry in pairs(g_currentMission.husbandries) do
			
			horse = animalCount(husbandry)
				
			for _, animals in pairs(husbandry:getAnimals()) do
				if animals.module.animalType ~= nil and animals.module.animalType == "HORSE" and checkIsOwnerOfFarm(husbandry) == true then
					c = c + 1
					-- get riding timer
					horseRideTimer = animals.ridingTimer or 0.00
					local horseTimerSent = animals.ridingTimerSent or 0.00
					local horseRideTarget = animals.DAILY_TARGET_RIDING_TIME or 300000
					local ridingToDo = horseRideTarget - horseRideTimer
					local horseCost = 0
				
					if horseRideTimer < horseRideTarget then
						-- add 25% to riding timer
						local newRideTime = tonumber(math.ceil(horseRideTimer + (horseRideTarget * horseDelta)))
						if newRideTime <= horseRideTarget then
							animals.ridingTimer = newRideTime
							animals.ridingTimerSent = newRideTime
							-- Pay for training per horse
							if horse > 0 and not self.payOnce then
								self.payOnce = true -- once per day
								horseCost = tonumber(math.ceil(horseRideRate * horse) * 4)
								if horseCost ~= 0 then
									g_currentMission:farmStats(farmData.farmId):updateStats("expenses", horseCost)
									g_currentMission:addMoney(-horseCost, farmData.farmId, MoneyType.ANIMAL_UPKEEP, true)
									nText = "Animal Worker: (-" .. horseCost .. ") " .. horse .. " horses trained."
									counter = nDur
									print("Animal Worker: Hourly horse training complete for " .. c .. " of " .. horse .. " horses, totaling " .. horseCost .. " for the day.")
									self.payOnce = false
									
								end
							end
						else
							print("New riding timer not sent. Value: " .. newRideTime .. ".")
						end
					else
						print("Animal Worker: Less than 25% riding left to do. Left: " ..ridingToDo .. ".")
					end
					
					
				end -- horses only
				
			end -- inner for loop end
		end -- outer horse for loop end
		self.doHorse = true -- run once each hour for 4 hours
	end
	
end

function animalWorker:draw()
end